require 'vr_tools/script/lua/vr_tools_flow_callbacks'

Project = Project or {}
VRSystem = require 'vr_tools/script/lua/vr_system'

Project.VRPlugin = Project.VRPlugin or {}

local VRPlugin = Project.VRPlugin

local World = stingray.World
local Unit = stingray.Unit
local Application = stingray.Application
local Matrix4x4 = stingray.Matrix4x4
local Window = stingray.Window
local Math = stingray.Math

function VRPlugin:init_vr_system()
	if self.vr_system then return true end
	if not VRSystem then return false end
	if not VRApi then return false end

	self.vr_system = VRSystem(self._world)

	self.vr_system:initialize(0.01, 1000.0, "vr_target", "vr_hud")
	if self.vr_system:is_initialized() then
		if not self.settings.skip_spawn_controllers then
			self.vr_system:spawn_controllers()
		end
		self.vr_system:enable()
		self:set_initial_tracking_space()

		Window.set_clip_cursor(false)
		Window.set_show_cursor(true)
			
		self.is_enabled = true
	else
		print("VR Initialization failed.")
		self.is_enabled = false
	end
	return self.is_enabled
end

function VRPlugin:set_initial_tracking_space()
	local player_start_pose = Appkit.PlayerUtil.get_player_start_pose(self._world) or stingray.Matrix4x4.identity()

	local start_position = Matrix4x4.translation(player_start_pose)
	local start_rotation = Matrix4x4.rotation(player_start_pose)
	local start_scale = Matrix4x4.scale(player_start_pose)

	VRApi.set_tracking_space(start_position, start_rotation, start_scale)
	if VRSystem.system_name == "oculus" then
		VRApi.set_tracking_origin(VRApi.TRACKING_ORIGIN_FLOOR_LEVEL)
	end
end

function VRPlugin:init()
	print("VRPLugin: init")
	self.vr_system = nil
	self.vr_input_mapper = nil
	self.is_enabled = false
	
	self.vr_player_height = 1.778
	self.vr_eye_height = 1.675
	self.settings = Application.settings() or {} -- script_data in ini
	self.vr_supported = Application.render_setting('vr_supported')
	return true
end

function VRPlugin:manage_world(world)
	if not self.vr_supported then return end
	print("VRPLugin: manage_world")
	self._world = world
end

function VRPlugin:update(dt)
	if self.vr_system and self.vr_system:is_enabled() and self._world then
		-- Update the Timpani Listener to include the Oculus device pose
		local tw = stingray.World.timpani_world(SimpleProject.world)
		if tw then
			local eye_poses = self.vr_system.hmd_world_pose(self.vr_system)
			tw:set_listener(1, eye_poses.left_eye)
		end
		return true
	end
end

function VRPlugin:shutdown()
	if self.vr_system then
		self.vr_system:shutdown()
		return true
	end
end

function VRPlugin:render()
	local managed_world = Appkit.managed_world
	if self.vr_system and self.vr_system:is_enabled() and self._world then
		return self.vr_system:render(managed_world.shading_env)
	end
end

return VRPlugin
