require 'core/appkit/lua/class'

VRSystem = VRSystem or nil
VRApi = VRApi or nil

if VRSystem then return VRSystem end

local Application = stingray.Application
local app_data = Application.settings() or {}

local vr_disabled = (app_data.vr_enabled == false)

local vr_render_supported = Application.render_setting('vr_supported')
if not vr_disabled and vr_render_supported then
	local system_name = app_data.vr_system

	if not system_name then
		if stingray.Oculus and stingray.Oculus.is_hmd_present() then
			system_name = "oculus"
		elseif stingray.SteamVR and stingray.SteamVR.is_hmd_present() then
			system_name = "steam"
		end
	end

	if system_name == "oculus" then
		-- Oculus System
		require 'vr_oculus/script/lua/oculus_vr_flow_callbacks'
		require 'vr_oculus/script/lua/oculus_vr'
		VRSystem = OculusVRSystem
		VRApi = stingray.Oculus
		VRSystem.system_name = "oculus"
	elseif system_name == "steam" then
		-- Steam System
		require 'vr_steam/script/lua/steam_vr_flow_callbacks'
		require 'vr_steam/script/lua/steam_vr'
		VRSystem = SteamVRSystem
		VRApi = stingray.SteamVR
		VRSystem.system_name = "steam"
	else
		VRSystem = {}
		function VRSystem:is_enabled()
			return false
		end
	end
end

return VRSystem
